class Sprite {

	boolean showArrow = true;
	PVector acceleration;
	PVector velocity;
	PVector location;
	boolean killMe = false;

	Sprite() {
		println("sprite constructor");
		
		acceleration = new PVector(0, 0);
		velocity = new PVector(0, 0);
		location = new PVector(random(width), 10);
	}
	
	void tick() {
		update();
		checkEdges();
		render();
	}
	
	void update() {
		velocity.add(acceleration);
		velocity.add(gravity);
		velocity.add(wind);		
		location.add(velocity);
		
		float perlin = perlinField.getNoiseValue((int)location.x, (int)location.y);
		
		PVector steering = new PVector(map(perlin , 0, 1, -.02, .02), 0);
		PVector friction = new PVector(0, map(perlin , 0, 1, -.05, .05));		
		acceleration.add(steering);
		acceleration.add(friction);		
		
		// don't let them go up
		if(acceleration.y < 0) acceleration.y = 0;
		
		acceleration.limit(.05);
		velocity.limit(5);		
	}
	
  void checkEdges() {
    if (location.y > height) {
      // mark for deletion
      killMe = true;
    } 
  }	
	
	void render() {
		// fill(255);
		// rect(location.x, location.y, 20, 20);
		if(showArrow) {
			drawVector(velocity, location, 10);
		}
		
	}
}


// Renders a vector object 'v' as an arrow and a location 'loc'
// From Dan Shiffman's example
void drawVector(PVector v, PVector loc, float scale) {
  pushMatrix();
  float arrowsize = 12;
  // Translate to location to render vector
  translate(loc.x,loc.y);
  stroke(255);
	strokeWeight(2);
	
  // Call vector heading function to get direction (note that pointing up is a heading of 0) and rotate
  rotate(v.heading2D());
  // Calculate length of vector & scale it to be bigger or smaller if necessary
  float len = v.mag()*scale;
  // Draw three lines to make an arrow (draw pointing up since we've rotate to the proper direction)
  line(0,0,len,0);
  line(len,0,len-arrowsize,+arrowsize/2);
  line(len,0,len-arrowsize,-arrowsize/2);
  popMatrix();
}
